@@ -5,6 +5,9 @@ The core submodule is the main suite of tools for PGx research.
 import pkgutil
 import pathlib
 from io import BytesIO
+import warnings
+
+from .. import sdk
 
 import numpy as np
 import pandas as pd
@@ -31,21 +34,6 @@ FUNCTION_ORDER = [
     'Class IV (Normal)',
 ]
 
-class AlleleNotFoundError(Exception):
-    """Raise if specified allele is not present in the allele table."""
-
-class GeneNotFoundError(Exception):
-    """Raise if specified gene is not present in the gene table."""
-
-class NotTargetGeneError(Exception):
-    """Raise if specified gene is not one of the target genes."""
-
-class PhenotypeNotFoundError(Exception):
-    """Raise if specified phenotype is not present in the phenotype table."""
-
-class VariantNotFoundError(Exception):
-    """Raise if specified variant is not present in the variant table."""
-
 def build_definition_table(gene, assembly='GRCh37'):
     """
     Build the definition table of star alleles for specified gene.
@@ -82,9 +70,7 @@ def build_definition_table(gene, assembly='GRCh37'):
     1    19  15897578  rs3093105   A   C    .      .   VI=W12G     GT  1  0
     """
     if not is_target_gene(gene):
-        raise NotTargetGeneError(gene)
-
-    other = 'GRCh38' if assembly == 'GRCh37' else 'GRCh37'
+        raise sdk.utils.NotTargetGeneError(gene)
 
     df1 = load_allele_table()
     df1 = df1[df1.Gene == gene]
@@ -133,7 +119,34 @@ def build_definition_table(gene, assembly='GRCh37'):
 
 def collapse_alleles(gene, alleles, assembly='GRCh37'):
     """
-    Collapse redundant candidate alleles.
+    Collapse redundant candidate star alleles.
+
+    Note that this method only considers core variants for collapsing.
+
+    Parameters
+    ----------
+    gene : str
+        Gene name.
+    alleles : list
+        List of alleles.
+    assembly : {'GRCh37', 'GRCh38'}, default: 'GRCh37'
+        Reference genome assembly.
+
+    Returns
+    -------
+    list
+        Collapsed list of alleles.
+
+    Examples
+    --------
+
+    >>> import pypgx
+    >>> pypgx.list_variants('CYP2B6', alleles='*6', mode='core')
+    ['19-41512841-G-T', '19-41515263-A-G']
+    >>> pypgx.list_variants('CYP2B6', alleles='*7', mode='core')
+    ['19-41512841-G-T', '19-41515263-A-G', '19-41522715-C-T']
+    >>> pypgx.collapse_alleles('CYP2B6', ['*6', '*7'])
+    ['*7']
     """
     results = []
     for a in alleles:
@@ -174,7 +187,7 @@ def has_phenotype(gene):
     False
     """
     if not is_target_gene(gene):
-        raise NotTargetGeneError(gene)
+        raise sdk.utils.NotTargetGeneError(gene)
 
     df = load_gene_table()
 
@@ -204,7 +217,7 @@ def has_score(gene):
     False
     """
     if not is_target_gene(gene):
-        raise NotTargetGeneError(gene)
+        raise sdk.utils.NotTargetGeneError(gene)
 
     df = load_gene_table()
 
@@ -297,9 +310,23 @@ def get_exon_ends(gene, assembly='GRCh37'):
     -------
     list
         List of end positions.
+
+    See Also
+    --------
+    get_exon_starts
+        Get exon starts for specified gene.
+
+    Examples
+    --------
+
+    >>> import pypgx
+    >>> pypgx.get_exon_ends('CYP2D6')
+    [42522754, 42522994, 42523636, 42523985, 42524352, 42524946, 42525187, 42525911, 42526883]
+    >>> pypgx.get_exon_ends('CYP2D6', assembly='GRCh38')
+    [42126752, 42126992, 42127634, 42127983, 42128350, 42128944, 42129185, 42129909, 42130810]
     """
     if gene not in list_genes(mode='all'):
-        raise GeneNotFoundError(gene)
+        raise sdk.utils.GeneNotFoundError(gene)
     df = load_gene_table()
     df = df[df.Gene == gene]
     s = df[f'{assembly}ExonEnds'].values[0]
@@ -320,9 +347,23 @@ def get_exon_starts(gene, assembly='GRCh37'):
     -------
     list
         List of start positions.
+
+    See Also
+    --------
+    get_exon_ends
+        Get exon ends for specified gene.
+
+    Examples
+    --------
+
+    >>> import pypgx
+    >>> pypgx.get_exon_starts('CYP2D6')
+    [42522500, 42522852, 42523448, 42523843, 42524175, 42524785, 42525034, 42525739, 42526613]
+    >>> pypgx.get_exon_starts('CYP2D6', assembly='GRCh38')
+    [42126498, 42126850, 42127446, 42127841, 42128173, 42128783, 42129032, 42129737, 42130611]
     """
     if gene not in list_genes(mode='all'):
-        raise GeneNotFoundError(gene)
+        raise sdk.utils.GeneNotFoundError(gene)
     df = load_gene_table()
     df = df[df.Gene == gene]
     s = df[f'{assembly}ExonStarts'].values[0]
@@ -360,13 +401,13 @@ def get_function(gene, allele):
     nan
     """
     if not is_target_gene(gene):
-        raise NotTargetGeneError(gene)
+        raise sdk.utils.NotTargetGeneError(gene)
 
     df = load_allele_table()
     df = df[(df.Gene == gene) & (df.StarAllele == allele)]
 
     if df.empty:
-        raise AlleleNotFoundError(gene, allele)
+        raise sdk.utils.AlleleNotFoundError(gene, allele)
 
     return df.Function.values[0]
 
@@ -430,10 +471,10 @@ def get_priority(gene, phenotype):
     'Normal/Routine/Low Risk'
     """
     if not is_target_gene(gene):
-        raise NotTargetGeneError(gene)
+        raise sdk.utils.NotTargetGeneError(gene)
 
     if phenotype not in list_phenotypes():
-        raise PhenotypeNotFoundError(phenotype)
+        raise sdk.utils.PhenotypeNotFoundError(phenotype)
 
     df = load_phenotype_table()
     i = (df.Gene == gene) & (df.Phenotype == phenotype)
@@ -448,8 +489,6 @@ def get_ref_allele(gene, assembly='GRCh37'):
     ----------
     gene : str
         Target gene.
-    assembly : {'GRCh37', 'GRCh38'}, default: 'GRCh37'
-        Reference genome assembly.
 
     Returns
     -------
@@ -486,7 +525,7 @@ def get_region(gene, assembly='GRCh37'):
         Requested region.
     """
     if gene not in list_genes(mode='all'):
-        raise GeneNotFoundError(gene)
+        raise sdk.utils.GeneNotFoundError(gene)
 
     df = load_gene_table()
     df = df[df.Gene == gene]
@@ -528,7 +567,7 @@ def get_score(gene, allele):
     nan
     """
     if not is_target_gene(gene):
-        raise NotTargetGeneError(gene)
+        raise sdk.utils.NotTargetGeneError(gene)
 
     if not has_score(gene):
         return np.nan
@@ -537,7 +576,7 @@ def get_score(gene, allele):
     df = df[(df.Gene == gene) & (df.StarAllele == allele)]
 
     if df.empty:
-        raise AlleleNotFoundError(gene, allele)
+        raise sdk.utils.AlleleNotFoundError(gene, allele)
 
     return df.ActivityScore.values[0]
 
@@ -556,7 +595,7 @@ def get_strand(gene):
         '+' or '-'.
     """
     if not is_target_gene(gene):
-        raise NotTargetGeneError(gene)
+        raise sdk.utils.NotTargetGeneError(gene)
 
     df = load_gene_table()
     df = df[df.Gene == gene]
@@ -587,17 +626,17 @@ def get_variant_impact(variant):
     'Splice Defect'
     >>> pypgx.get_variant_impact('22-42524435-T-A') # Intron variant
     ''
-    >>> pypgx.get_variant_impact('22-42524435-T-C') # Does not exist
+    >>> pypgx.get_variant_impact('22-42524435-T-C')
     Traceback (most recent call last):
       File "<stdin>", line 1, in <module>
-      File "/Users/sbslee/Desktop/pypgx/pypgx/api/core.py", line 489, in get_variant_impact
-        raise VariantNotFoundError(variant)
-    pypgx.api.core.VariantNotFoundError: 22-42524435-T-C
+      File "/Users/sbslee/Desktop/pypgx/pypgx/api/core.py", line 588, in get_variant_impact
+        raise sdk.utils.VariantNotFoundError(variant)
+    pypgx.sdk.utils.VariantNotFoundError: 22-42524435-T-C
     """
     df = load_variant_table()
     df = df[(df.GRCh37Name == variant) | (df.GRCh38Name == variant)]
     if df.empty:
-        raise VariantNotFoundError(variant)
+        raise sdk.utils.VariantNotFoundError(variant)
     impact = df.Impact.values[0]
     if pd.isna(impact):
         impact = ''
@@ -634,7 +673,8 @@ def get_variant_synonyms(gene, assembly='GRCh37'):
     for i, r in df.iterrows():
         if pd.isna(r[f'{assembly}Synonym']):
             continue
-        synonyms[r[f'{assembly}Synonym']] = r[f'{assembly}Name']
+        for variant in r[f'{assembly}Synonym'].split(','):
+            synonyms[variant] = r[f'{assembly}Name']
     return synonyms
 
 def list_alleles(gene, variants=None, assembly='GRCh37'):
@@ -782,7 +822,7 @@ def list_phenotypes(gene=None):
 
     if gene is not None:
         if not is_target_gene(gene):
-            raise NotTargetGeneError(gene)
+            raise sdk.utils.NotTargetGeneError(gene)
         df = df[df.Gene == gene]
 
     return sorted(list(df.Phenotype.unique()))
@@ -833,7 +873,7 @@ def list_variants(gene, alleles=None, mode='all', assembly='GRCh37'):
     ['19-41495755-T-C', '19-41496461-T-C']
     """
     if not is_target_gene(gene):
-        raise NotTargetGeneError(gene)
+        raise sdk.utils.NotTargetGeneError(gene)
 
     allele_table = load_allele_table()
     allele_table = allele_table[allele_table.Gene == gene]
@@ -851,7 +891,7 @@ def list_variants(gene, alleles=None, mode='all', assembly='GRCh37'):
         df = allele_table[allele_table.StarAllele == allele]
 
         if df.empty:
-            raise AlleleNotFoundError(gene, allele)
+            raise sdk.utils.AlleleNotFoundError(gene, allele)
 
         c = df[f'{assembly}Core'].values[0]
         t = df[f'{assembly}Tag'].values[0]
@@ -887,6 +927,13 @@ def load_allele_table():
 
     >>> import pypgx
     >>> df = pypgx.load_allele_table()
+    >>> df.head()
+          Gene StarAllele  ActivityScore                           Function                                    GRCh37Core GRCh37Tag                                    GRCh38Core GRCh38Tag     SV
+    0    ABCB1         *1            NaN                    Normal Function  7-87138645-A-G,7-87160618-A-C,7-87179601-A-G       NaN  7-87509329-A-G,7-87531302-A-C,7-87550285-A-G       NaN  False
+    1    ABCB1         *2            NaN                 Increased Function                                           NaN       NaN                                           NaN       NaN  False
+    2  CACNA1S  Reference            NaN                    Normal Function                                           NaN       NaN                                           NaN       NaN  False
+    3  CACNA1S   c.520C>T            NaN  Malignant Hyperthermia Associated                               1-201061121-G-A       NaN                               1-201091993-G-A       NaN  False
+    4  CACNA1S  c.3257G>A            NaN  Malignant Hyperthermia Associated                               1-201029943-C-T       NaN                               1-201060815-C-T       NaN  False
     """
     b = BytesIO(pkgutil.get_data(__name__, 'data/allele-table.csv'))
     return pd.read_csv(b)
@@ -905,10 +952,42 @@ def load_cnv_table():
 
     >>> import pypgx
     >>> df = pypgx.load_cnv_table()
+    >>> df.head()
+         Gene          Name
+    0  CYP2A6        Normal
+    1  CYP2A6  Deletion1Het
+    2  CYP2A6  Deletion1Hom
+    3  CYP2A6  Deletion2Het
+    4  CYP2A6  Deletion3Het
     """
     b = BytesIO(pkgutil.get_data(__name__, 'data/cnv-table.csv'))
     return pd.read_csv(b)
 
+def load_cpic_table():
+    """
+    Load the CPIC table.
+
+    Returns
+    -------
+    pandas.DataFrame
+        Requested table.
+
+    Examples
+    --------
+
+    >>> import pypgx
+    >>> df = pypgx.load_cpic_table()
+    >>> df.head()
+          Gene           Drug                                          Guideline CPICLevel CPICLevelStatus PharmGKBLevel             FDALabel               PMID
+    0    HLA-B       abacavir  https://cpicpgx.org/guidelines/guideline-for-a...         A           Final            1A     Testing required  24561393;22378157
+    1    HLA-B    allopurinol  https://cpicpgx.org/guidelines/guideline-for-a...         A           Final            1A  Testing recommended  23232549;26094938
+    2  MT-RNR1       amikacin  https://cpicpgx.org/guidelines/cpic-guideline-...         A           Final             3                  NaN           34032273
+    3  CYP2C19  amitriptyline  https://cpicpgx.org/guidelines/guideline-for-t...         A           Final            1A                  NaN  23486447;27997040
+    4   CYP2D6  amitriptyline  https://cpicpgx.org/guidelines/guideline-for-t...         A           Final            1A       Actionable PGx  23486447;27997040
+    """
+    b = BytesIO(pkgutil.get_data(__name__, 'data/cpic-table.csv'))
+    return pd.read_csv(b)
+
 def load_diplotype_table():
     """
     Load the diplotype table.
@@ -923,6 +1002,13 @@ def load_diplotype_table():
 
     >>> import pypgx
     >>> df = pypgx.load_diplotype_table()
+    >>> df.head()
+          Gene            Diplotype                              Phenotype
+    0  CACNA1S  Reference/Reference               Uncertain Susceptibility
+    1  CACNA1S   Reference/c.520C>T  Malignant Hyperthermia Susceptibility
+    2  CACNA1S  Reference/c.3257G>A  Malignant Hyperthermia Susceptibility
+    3  CACNA1S    c.520C>T/c.520C>T  Malignant Hyperthermia Susceptibility
+    4  CACNA1S   c.520C>T/c.3257G>A  Malignant Hyperthermia Susceptibility
     """
     b = BytesIO(pkgutil.get_data(__name__, 'data/diplotype-table.csv'))
     return pd.read_csv(b)
@@ -941,6 +1027,13 @@ def load_equation_table():
 
     >>> import pypgx
     >>> df = pypgx.load_equation_table()
+    >>> df.head()
+         Gene                 Phenotype              Equation
+    0  CYP2C9          Poor Metabolizer        0 <= score < 1
+    1  CYP2C9  Intermediate Metabolizer        1 <= score < 2
+    2  CYP2C9        Normal Metabolizer            2 == score
+    3  CYP2D6          Poor Metabolizer     0 <= score < 0.25
+    4  CYP2D6  Intermediate Metabolizer  0.25 <= score < 1.25
     """
     b = BytesIO(pkgutil.get_data(__name__, 'data/equation-table.csv'))
     return pd.read_csv(b)
@@ -959,6 +1052,13 @@ def load_gene_table():
 
     >>> import pypgx
     >>> df = pypgx.load_gene_table()
+    >>> df.head()
+          Gene  Target  Control Paralog  Variants     SV PhenotypeMethod  RefAllele GRCh37Default GRCh38Default Strand           GRCh37Region           GRCh38Region                                   GRCh37ExonStarts                                     GRCh37ExonEnds                                   GRCh38ExonStarts                                     GRCh38ExonEnds
+    0    ABCB1    True    False     NaN      True  False             NaN         *1            *2            *2      -    7:87130178-87345639    7:87500862-87716323  87133178,87135212,87138590,87144546,87145824,8...  87133765,87135359,87138797,87144744,87145981,8...  87503862,87505896,87509274,87515230,87516508,8...  87504449,87506043,87509481,87515428,87516665,8...
+    1  CACNA1S    True    False     NaN      True  False       Diplotype  Reference     Reference     Reference      -  1:201005639-201084694  1:201036511-201115426  201008639,201009358,201009749,201010631,201012...  201009210,201009502,201009841,201010717,201012...  201039511,201040230,201040621,201041503,201043...  201040082,201040374,201040713,201041589,201043...
+    2     CFTR    True    False     NaN      True  False       Diplotype  Reference     Reference     Reference      +  7:117117016-117311719  7:117477024-117671665  117120016,117144306,117149087,117170952,117174...  117120201,117144417,117149196,117171168,117174...  117480024,117504252,117509033,117530898,117534...  117480147,117504363,117509142,117531114,117534...
+    3   CYP1A1    True    False     NaN      True  False             NaN         *1            *1            *1      -   15:75008882-75020951   15:74716541-74728528  75011882,75013307,75013539,75013754,75013931,7...  75013115,75013394,75013663,75013844,75014058,7...  74719541,74720966,74721198,74721413,74721590,7...  74720774,74721053,74721322,74721503,74721717,7...
+    4   CYP1A2    True    False     NaN      True  False             NaN        *1A           *1A           *1A      +   15:75038183-75051941   15:74745844-74759607  75041183,75042070,75043529,75044105,75044464,7...  75041238,75042910,75043650,75044195,75044588,7...  74748844,74749729,74751188,74751764,74752123,7...  74748897,74750569,74751309,74751854,74752247,7...
     """
     b = BytesIO(pkgutil.get_data(__name__, 'data/gene-table.csv'))
     return pd.read_csv(b)
@@ -977,6 +1077,13 @@ def load_phenotype_table():
 
     >>> import pypgx
     >>> df = pypgx.load_phenotype_table()
+    >>> df.head()
+          Gene                              Phenotype                     Priority
+    0  CACNA1S               Uncertain Susceptibility                  Normal Risk
+    1  CACNA1S  Malignant Hyperthermia Susceptibility  Abnormal/Priority/High Risk
+    2     CFTR                     Favorable Response                         None
+    3     CFTR                   Unfavorable Response                         None
+    4     CFTR                          Indeterminate                         None
     """
     b = BytesIO(pkgutil.get_data(__name__, 'data/phenotype-table.csv'))
     return pd.read_csv(b)
@@ -995,6 +1102,13 @@ def load_variant_table():
 
     >>> import pypgx
     >>> df = pypgx.load_phenotype_table()
+    >>> df.head()
+          Gene                              Phenotype                     Priority
+    0  CACNA1S               Uncertain Susceptibility                  Normal Risk
+    1  CACNA1S  Malignant Hyperthermia Susceptibility  Abnormal/Priority/High Risk
+    2     CFTR                     Favorable Response                         None
+    3     CFTR                   Unfavorable Response                         None
+    4     CFTR                          Indeterminate                         None
     """
     b = BytesIO(pkgutil.get_data(__name__, 'data/variant-table.csv'))
     df = pd.read_csv(b)
@@ -1008,6 +1122,10 @@ def predict_phenotype(gene, a, b):
     The method can handle star alleles with structural variation including
     gene deletion, duplication, and tandem arrangement.
 
+    For detailed implementation, please see the `Phenotype prediction
+    <file:///Users/sbslee/Desktop/pypgx/docs/_build/html/
+    readme.html#phenotype-prediction>`__ section.
+
     Parameters
     ----------
     gene : str
@@ -1036,7 +1154,7 @@ def predict_phenotype(gene, a, b):
     'Rapid Metabolizer'
     """
     if not is_target_gene(gene):
-        raise NotTargetGeneError(gene)
+        raise sdk.utils.NotTargetGeneError(gene)
 
     df = load_gene_table()
     phenotype_method = df[df.Gene == gene].PhenotypeMethod.values[0]
@@ -1055,9 +1173,9 @@ def predict_phenotype(gene, a, b):
         df = load_diplotype_table()
         df = df[df.Gene == gene]
         if not is_legit_allele(gene, a):
-            raise AlleleNotFoundError(gene, a)
+            warnings.warn(f"{a} not found in the allele table for {gene}")
         if not is_legit_allele(gene, b):
-            raise AlleleNotFoundError(gene, b)
+            warnings.warn(f"{b} not found in the allele table for {gene}")
         l = [f'{a}/{b}', f'{b}/{a}']
         i = df.Diplotype.isin(l)
         try:
@@ -1080,6 +1198,10 @@ def predict_score(gene, allele):
     function as well as for alleles from a gene that does not use the
     activity score system.
 
+    For detailed implementation, please see the `Phenotype prediction
+    <file:///Users/sbslee/Desktop/pypgx/docs/_build/html/
+    readme.html#phenotype-prediction>`__ section.
+
     Parameters
     ----------
     gene : str
@@ -1122,14 +1244,16 @@ def predict_score(gene, allele):
     >>> pypgx.predict_score('CYP2D6', '*1x2+*4x2+*10') # Complex event
     2.25
 
-    We can also predict phenotypes for the DPYD gene:
+    We can also predict activity score for the DPYD gene:
 
-    >>> pypgx.predict_phenotype('DPYD', 'Reference', 'Reference')
-    'Normal Metabolizer'
-    >>> pypgx.predict_phenotype('DPYD', 'Reference', 'c.1905+1G>A (*2A)')
-    'Intermediate Metabolizer'
-    >>> pypgx.predict_phenotype('DPYD', 'c.295_298delTCAT (*7)', 'c.703C>T (*8)')
-    'Poor Metabolizer'
+    >>> pypgx.predict_score('DPYD', 'Reference')
+    1.0
+    >>> pypgx.predict_score('DPYD', 'c.1905+1G>A (*2A)')
+    0.0
+    >>> pypgx.predict_score('DPYD', 'c.295_298delTCAT (*7)')
+    0.0
+    >>> pypgx.predict_score('DPYD', 'c.703C>T (*8)')
+    0.0
 
     All of the CYP2B6 alleles will return ``NaN`` because it does not have activity score:
 
@@ -1139,7 +1263,7 @@ def predict_score(gene, allele):
     nan
     """
     if not is_target_gene(gene):
-        raise NotTargetGeneError(gene)
+        raise sdk.utils.NotTargetGeneError(gene)
 
     if not has_score(gene):
         return np.nan
@@ -1176,11 +1300,12 @@ def sort_alleles(
     variants (e.g. three SNVs > one SNV), 3. number of core variants that
     impact protein coding (e.g. two misssense variants > one missense variant
     plus one intron variant), and 4. reference allele status (e.g.
-    non-reference allele with one SNV > reference allele with one SNV).
-    Note that the priority of allele function decreases in the following
-    order: 'No Function', 'Decreased Function', 'Possible Decreased
-    Function', 'Increased Function', 'Possible Increased Function',
-    'Uncertain Function', 'Unknown Function', 'Normal Function'.
+    non-reference allele with two SNVs > reference allele with two SNVs such
+    that CYP2D6\*46 > CYP2D6\*1 in GRCh37). Note that the priority of allele
+    function decreases in the following order: 'No Function', 'Decreased
+    Function', 'Possible Decreased Function', 'Increased Function', 'Possible
+    Increased Function', 'Uncertain Function', 'Unknown Function', 'Normal
+    Function'.
 
     When ``by='name'`` the method will report alleles with a smaller
     number first. This means, for example, '\*4' will come before '\*10'
@@ -1239,7 +1364,7 @@ def sort_alleles(
         if gene is None:
             raise ValueError('Gene is required when sorting by priority')
         if not is_target_gene(gene):
-            raise NotTargetGeneError(gene)
+            raise sdk.utils.NotTargetGeneError(gene)
         function = get_function(gene, allele)
         a = FUNCTION_ORDER.index(function)
         core_variants = list_variants(gene, alleles=allele, assembly=assembly, mode='core')
