@@ -37,6 +37,10 @@ def _call_duplication(r):
             result = [a2, a1+'x2']
         elif not r.VariantData[a1] and r.VariantData[a2]:
             result = [a1, a2+'x2']
+        elif a1 in r.Haplotype2:
+            result = [a1, a2+'x2']
+        elif a2 in r.Haplotype1:
+            result = [a2, a1+'x2']
         else:
             result = ['Indeterminate']
     elif h1 and not h2:
@@ -82,6 +86,23 @@ def _call_multiplication(r):
 
     return result
 
+def _call_linked_allele(r, linked, target):
+    """
+    Call linked star allele.
+    """
+    a1, a2 = r.Haplotype1[0], r.Haplotype2[0]
+    h1 = linked in r.Haplotype1
+    h2 = linked in r.Haplotype2
+    if h1 and h2:
+        result = [a1, target]
+    elif h1 and not h2:
+        result = [a2, target]
+    elif not h1 and h2:
+        result = [a1, target]
+    else:
+        result = ['Indeterminate']
+    return result
+
 ###############################
 # Public classes and methods  #
 ###############################
@@ -108,20 +129,20 @@ class CYP2A6Genotyper:
 
     def one_row(self, r):
         a1, a2 = r.Haplotype1[0], r.Haplotype2[0]
-        if r.CNV in ['Normal', 'AssumeNormal']:
+        s1, s2 = core.sort_alleles([a1, a2], by='priority', gene=self.gene, assembly=self.assembly)
+        if r.CNV in ['Normal', 'AssumeNormal', 'PseudogeneDuplication']:
             result = [a1, a2]
-        elif r.CNV == 'DeletionHom':
+        elif r.CNV == 'Deletion1Hom':
             result = ['*4', '*4']
-        elif r.CNV == 'DeletionHet':
-            if a1 == a2:
-                result = [a1, '*4']
-            elif a1 == '*1':
-                result = [a2, '*4']
-            elif a2 == '*1':
-                result = [a1, '*4']
-            else:
-                result = ['Indeterminate']
-        elif r.CNV == 'Duplication':
+        elif r.CNV in ['Deletion1Het', 'Deletion2Het', 'Deletion3Het']:
+            result = [s1, '*4']
+        elif r.CNV == 'Hybrid2':
+            result = [s1, '*12']
+        elif r.CNV == 'Hybrid2Hom':
+            result = ['*12', '*12']
+        elif r.CNV == 'Hybrid3':
+            result = [s1, '*34']
+        elif r.CNV in ['Duplication1', 'Duplication2', 'Duplication3']:
             result = _call_duplication(r)
         else:
             result = ['Indeterminate']
@@ -145,6 +166,8 @@ class CYP2B6Genotyper:
             result = [a1, a2]
         elif r.CNV == 'Hybrid':
             result = [p, '*29']
+        elif r.CNV == 'Duplication':
+            result = _call_duplication(r)
         else:
             result = ['Indeterminate']
         return '/'.join(core.sort_alleles(result, by='name'))
@@ -162,7 +185,7 @@ class CYP2D6Genotyper:
     def one_row(self, r):
         a1, a2 = r.Haplotype1[0], r.Haplotype2[0]
         s1, s2 = core.sort_alleles([a1, a2], by='priority', gene=self.gene, assembly=self.assembly)
-        if r.CNV in ['Normal', 'AssumeNormal']:
+        if r.CNV in ['Normal', 'AssumeNormal', 'PseudogeneDeletion']:
             result = [a1, a2]
         elif r.CNV == 'DeletionHom':
             result = ['*5', '*5']
@@ -170,7 +193,9 @@ class CYP2D6Genotyper:
             result = [s1, '*5']
         elif r.CNV == 'Duplication':
             result = _call_duplication(r)
-        elif r.CNV == 'Tandem1':
+        elif r.CNV == 'Multiplication':
+            result = _call_multiplication(r)
+        elif r.CNV == 'Tandem1A':
             h1 = '*4' in r.Haplotype1
             h2 = '*4' in r.Haplotype2
             if h1 and h2:
@@ -181,6 +206,17 @@ class CYP2D6Genotyper:
                 result = [a1, '*68+*4']
             else:
                 result = ['Indeterminate']
+        elif r.CNV == 'Tandem1B':
+            h1 = '*4' in r.Haplotype1
+            h2 = '*4' in r.Haplotype2
+            if h1 and h2:
+                result = ['*68+*4', '*68+*4']
+            elif h1 and not h2:
+                result = [a2, '*68x2+*4']
+            elif not h1 and h2:
+                result = [a1, '*68x2+*4']
+            else:
+                result = ['Indeterminate']
         elif r.CNV == 'Tandem2A':
             h1 = '*10' in r.Haplotype1
             h2 = '*10' in r.Haplotype2
@@ -225,12 +261,12 @@ class CYP2D6Genotyper:
                 result = [a1, '*13+*1']
             else:
                 result = ['Indeterminate']
-        elif 'DeletionHet' in r.CNV and 'Tandem1' in r.CNV:
+        elif 'DeletionHet' in r.CNV and 'Tandem1A' in r.CNV:
             if '*4' in a1 or '*4' in a2:
                 result = ['*5', '*68+*4']
             else:
                 result = ['Indeterminate']
-        elif 'Duplication' in r.CNV and 'Tandem1' in r.CNV:
+        elif 'Duplication' in r.CNV and 'Tandem1A' in r.CNV:
             h1 = '*4' in r.Haplotype1
             h2 = '*4' in r.Haplotype2
             if h1 and h2:
@@ -260,9 +296,9 @@ class CYP2E1Genotyper:
         a1, a2 = r.Haplotype1[0], r.Haplotype2[0]
         if r.CNV in ['Normal', 'AssumeNormal']:
             result = [a1, a2]
-        elif r.CNV == 'PartialDuplication':
-            h1 = '*4'in r.Haplotype1 and '*7' in r.Haplotype1
-            h2 = '*4'in r.Haplotype2 and '*7' in r.Haplotype2
+        elif r.CNV == 'PartialDuplicationHet':
+            h1 = '*7' in r.Haplotype1
+            h2 = '*7' in r.Haplotype2
             if h1 and h2:
                 result = [a1, '*S1']
             elif h1 and not h2:
@@ -271,7 +307,9 @@ class CYP2E1Genotyper:
                 result = [a1, '*S1']
             else:
                 result = ['Indeterminate']
-        elif r.CNV == 'Duplication':
+        elif r.CNV == 'PartialDuplicationHom':
+            result = ['*S1', '*S1']
+        elif r.CNV in ['Duplication1', 'Duplication2']:
             result = _call_duplication(r)
         elif r.CNV == 'Multiplication':
             result = _call_multiplication(r)
@@ -305,6 +343,27 @@ class CYP4F2Genotyper:
         self.assembly = assembly
         self.results = df.apply(self.one_row, axis=1)
 
+class G6PDGenotyper:
+    """
+    Genotyper for G6PD.
+    """
+
+    def one_row(self, r):
+        a1, a2 = r.Haplotype1[0], r.Haplotype2[0]
+        s1, s2 = core.sort_alleles([a1, a2], by='priority', gene=self.gene, assembly=self.assembly)
+        if r.CNV in ['Female', 'AssumeNormal']:
+            result = [a1, a2]
+        elif r.CNV == 'Male':
+            result = [s1, '*MALE']
+        else:
+            result = ['Indeterminate']
+        return '/'.join(core.sort_alleles(result, by='name'))
+
+    def __init__(self, df, assembly):
+        self.gene = 'G6PD'
+        self.assembly = assembly
+        self.results = df.apply(self.one_row, axis=1)
+
 class GSTM1Genotyper:
     """
     Genotyper for GSTM1.
@@ -312,17 +371,15 @@ class GSTM1Genotyper:
 
     def one_row(self, r):
         a1, a2 = r.Haplotype1[0], r.Haplotype2[0]
-        if r.CNV == 'DeletionHet':
-            if a1 == a2:
-                result = [a1, '*0']
-            else:
-                result = ['Indeterminate']
-        elif r.CNV == 'DeletionHom':
+        s1, s2 = core.sort_alleles([a1, a2], by='priority', gene=self.gene, assembly=self.assembly)
+        if r.CNV in ['Normal', 'AssumeNormal', 'UpstreamDeletionHet']:
+            result = [a1, a2]
+        elif r.CNV in ['DeletionHet', 'DeletionHet,UpstreamDeletionHet']:
+            result = [s1, '*0']
+        elif r.CNV in ['DeletionHom', 'DeletionHet,Deletion2']:
             result = ['*0', '*0']
         elif r.CNV == 'Duplication':
             result = _call_duplication(r)
-        elif r.CNV in ['Normal', 'AssumeNormal']:
-            result = [a1, a2]
         else:
             result = ['Indeterminate']
         return '/'.join(core.sort_alleles(result, by='name'))
@@ -384,6 +441,12 @@ class SLC22A2Genotyper:
                 result = [a1, '*S2']
             else:
                 result = ['Indeterminate']
+        elif r.CNV == 'Intron9Deletion,Exon11Deletion':
+            if (('*3' in r.Haplotype1 or '*3' in r.Haplotype2) and
+                ('*K432Q' in r.Haplotype1 or '*K432Q' in r.Haplotype2)):
+                result = ['*S1', '*S2']
+            else:
+                result = ['Indeterminate']
         else:
             result = ['Indeterminate']
         return '/'.join(core.sort_alleles(result, by='name'))
@@ -405,6 +468,8 @@ class SULT1A1Genotyper:
             result = [a1, a2]
         elif r.CNV == 'DeletionHet':
             result = [s1, '*DEL']
+        elif r.CNV == 'DeletionHom':
+            result = ['*DEL', '*DEL']
         elif r.CNV == 'Duplication':
             result = _call_duplication(r)
         elif r.CNV == 'Multiplication1':
@@ -430,21 +495,11 @@ class UGT1A4Genotyper:
         if r.CNV in ['Normal', 'AssumeNormal']:
             result = [a1, a2]
         elif r.CNV == 'Intron1DeletionA':
-            if a1 == a2:
-                result = [a1, '*S1']
-            else:
-                result = ['Indeterminate']
+            result = _call_linked_allele(r, '*1', '*S1')
         elif r.CNV == 'Intron1DeletionB':
-            h1 = '*1'in r.Haplotype1
-            h2 = '*1'in r.Haplotype2
-            if h1 and h2:
-                result = [a1, '*S2']
-            elif h1 and not h2:
-                result = [a2, '*S2']
-            elif not h1 and h2:
-                result = [a1, '*S2']
-            else:
-                result = ['Indeterminate']
+            result = _call_linked_allele(r, '*1', '*S2')
+        elif r.CNV == 'Intron1PartialDup':
+            result = _call_linked_allele(r, '*1', '*S3')
         else:
             result = ['Indeterminate']
         return '/'.join(core.sort_alleles(result, by='name'))
@@ -461,13 +516,17 @@ class UGT2B15Genotyper:
 
     def one_row(self, r):
         a1, a2 = r.Haplotype1[0], r.Haplotype2[0]
+        s1, s2 = core.sort_alleles([a1, a2], by='priority', gene=self.gene, assembly=self.assembly)
         if r.CNV in ['Normal', 'AssumeNormal']:
             result = [a1, a2]
-        elif r.CNV == 'PartialDeletion':
-            if a1 == a2:
-                result = [a1, '*S1']
-            else:
-                result = ['Indeterminate']
+        elif r.CNV == 'PartialDeletion1':
+            result = [a1, '*S1']
+        elif r.CNV == 'PartialDeletion2':
+            result = [a1, '*S2']
+        elif r.CNV == 'PartialDeletion3':
+            result = [a1, '*S3']
+        elif r.CNV == 'Deletion':
+            result = [a1, '*S4']
         else:
             result = ['Indeterminate']
         return '/'.join(core.sort_alleles(result, by='name'))
@@ -483,12 +542,16 @@ class UGT2B17Genotyper:
     """
 
     def one_row(self, r):
-        if r.CNV == 'DeletionHet':
+        if r.CNV == 'Normal,Deletion':
             result = ['*1', '*2']
-        elif r.CNV == 'DeletionHom':
+        elif r.CNV == 'Deletion,Deletion':
             result = ['*2', '*2']
-        elif r.CNV in ['Normal', 'AssumeNormal']:
+        elif r.CNV in ['Normal,Normal', 'AssumeNormal']:
             result = ['*1', '*1']
+        elif r.CNV == 'Deletion,PartialDeletion1':
+            result = ['*2', '*S1']
+        elif r.CNV == 'Deletion,PartialDeletion2':
+            result = ['*2', '*S2']
         else:
             result = ['Indeterminate']
         return '/'.join(core.sort_alleles(result, by='name'))
@@ -520,6 +583,7 @@ def call_genotypes(alleles=None, cnv_calls=None):
         'CYP2D6': CYP2D6Genotyper,
         'CYP2E1': CYP2E1Genotyper,
         'CYP4F2': CYP4F2Genotyper,
+        'G6PD': G6PDGenotyper,
         'GSTM1': GSTM1Genotyper,
         'GSTT1': GSTT1Genotyper,
         'SLC22A2': SLC22A2Genotyper,
