@@ -5,30 +5,36 @@ from ..api import genotype
 import fuc
 
 description = f"""
-Call genotypes for the target gene.
+Call genotypes for target gene.
 """
 
 def create_parser(subparsers):
     parser = fuc.api.common._add_parser(
         subparsers,
         fuc.api.common._script_name(),
-        help='Call genotypes for the target gene.',
         description=description,
+        help=
+"""Call genotypes for target gene."""
     )
     parser.add_argument(
         'genotypes',
-        help='Archive file with the semantic type \n'
-             'SampleTable[Genotypes].'
+        help=
+"""Output archive file with the semantic type
+SampleTable[Genotypes]."""
     )
     parser.add_argument(
         '--alleles',
         metavar='PATH',
-        help='Archive file with the semantic type SampleTable[Alleles].'
+        help=
+"""Input archive file with the semantic type
+SampleTable[Alleles]."""
     )
     parser.add_argument(
         '--cnv-calls',
         metavar='PATH',
-        help='Archive file with the semantic type SampleTable[CNVCalls].'
+        help=
+"""Input archive file with the semantic type
+SampleTable[CNVCalls]."""
     )
 
 def main(args):
