@@ -5,43 +5,50 @@ from ..api import utils
 import fuc
 
 description = f"""
-Combine various results for the target gene.
+Combine various results for target gene.
 """
 
 def create_parser(subparsers):
     parser = fuc.api.common._add_parser(
         subparsers,
         fuc.api.common._script_name(),
-        help='Combine various results for the target gene.',
         description=description,
+        help=
+"""Combine various results for target gene."""
     )
     parser.add_argument(
         'results',
-        help='Archive file with the semantic type SampleTable[Results].'
+        help=
+"""Output archive file with the semantic type
+SampleTable[Results]."""
     )
     parser.add_argument(
         '--genotypes',
         metavar='PATH',
-        help='Archive file with the semantic type \n'
-             'SampleTable[Genotypes].'
+        help=
+"""Input archive file with the semantic type
+SampleTable[Genotypes]."""
     )
     parser.add_argument(
         '--phenotypes',
         metavar='PATH',
-        help='Archive file with the semantic type \n'
-             'SampleTable[Phenotypes].'
+        help=
+"""Input archive file with the semantic type
+SampleTable[Phenotypes]."""
     )
     parser.add_argument(
         '--alleles',
         metavar='PATH',
-        help='Archive file with the semantic type \n'
-             'SampleTable[Alleles].'
+        help=
+"""Input archive file with the semantic type
+SampleTable[Alleles]."""
     )
     parser.add_argument(
         '--cnv-calls',
         metavar='PATH',
-        help='Archive file with the semantic type \n'
-             'SampleTable[CNVCalls].'
+        help=
+"""Input archive file with the semantic type
+SampleTable[CNVCalls]."""
     )
 
 def main(args):
