@@ -7,10 +7,6 @@ import pysam
 
 description = f"""
 Estimate haplotype phase of observed variants with the Beagle program.
-
-Note that the 'chr' prefix in contig names (e.g. 'chr1' vs. '1') will be
-automatically added or removed as necessary to match the reference VCF’s
-contig names.
 """
 
 def create_parser(subparsers):
@@ -18,30 +14,40 @@ def create_parser(subparsers):
         subparsers,
         fuc.api.common._script_name(),
         description=description,
-        help='Estimate haplotype phase of observed variants with the '
-             'Beagle program.',
+        help=
+"""Estimate haplotype phase of observed variants with
+the Beagle program."""
     )
     parser.add_argument(
         'imported_variants',
         metavar='imported-variants',
-        help='Archive file with the semantic type VcfFrame[Imported].'
+        help=
+"""Input archive file with the semantic type
+VcfFrame[Imported]. The 'chr' prefix in contig names
+(e.g. 'chr1' vs. '1') will be automatically added or
+removed as necessary to match the reference VCF's contig
+names."""
     )
     parser.add_argument(
         'phased_variants',
         metavar='phased-variants',
-        help='Archive file with the semantic type VcfFrame[Phased].'
+        help=
+"""Output archive file with the semantic type
+VcfFrame[Phased]."""
     )
     parser.add_argument(
         '--panel',
         metavar='PATH',
-        help='VCF file corresponding to a reference haplotype panel \n'
-             '(compressed or uncompressed). By default, the 1KGP panel \n'
-             'is used.'
+        help=
+"""VCF file (compressed or uncompressed) corresponding to a
+reference haplotype panel. By default, the 1KGP panel in
+the ~/pypgx-bundle directory will be used."""
     )
     parser.add_argument(
         '--impute',
         action='store_true',
-        help='Perform imputation of missing genotypes.'
+        help=
+"""Perform imputation of missing genotypes."""
     )
 
 def main(args):
